function main(edgefile, markedset)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% write the marked set file %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dlmwrite('marked-nodes.txt',markedset','delimiter',' ','precision','%6.0f');

whichgrphs = 1:4;

% build the undirected unweighted adjacency matrix A
[src dst w] = textread(edgefile);
N = max([src; dst])
log2(N)

A = sparse(src, dst, ones(length(src),1), N, N);
A = A+A';
A(A>0) = 1;
 
% visualize the graph
if(N<=100)
bg = visGraph(A, [], markedset,0,0);
bg.view;
end

fprintf('There are %d edges in the graph...\n', full(sum(sum(A))/2));
% weight the graph
deg = sum(A,2);
logdeg = log2(deg+1);

    
W = (A'*spdiags(logdeg, 0, N, N))';

% prune graph --comment out for speed up
tic
PA = PruneGraph(A, markedset);
W = W.*PA;
toc

clear A;

if(sum(whichgrphs==2|whichgrphs==3|whichgrphs==4)>0)

    tStart = tic;
    [markedpaths] = ShortPaths2( W, logdeg, markedset );
    tElapsed = toc(tStart)
    disp('markedpaths computed!')
    %save('markedpaths.mat','markedpaths','-v7.3')       
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% TREE-1: CONNECTED COMPONENTS %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(sum(whichgrphs==1)>0)
tStart = tic;
[ PMIN, roots, cost ] = findConnCompT( W, markedset );
tCCelapsed = toc(tStart)

[trash, PMIN, trash, trash, trash] = expandPathsAll(W(markedset,markedset), PMIN(markedset,markedset), markedpaths, markedset, markedset, W, 1);

%PMIN
fprintf('Cost-alg from CONN-COMPS is %f \n', cost);
cost = findTreeCost( PMIN, markedset, markedset(roots), deg );
fprintf('Cost-act from CONN-COMPS is %f \n', cost);
%cost = findTreeCostJ( PMIN, markedset, markedset(roots), deg );
%fprintf('Cost-J from CONN-COMPS is %f \n', cost);

% write tree-1
[src dst trash] = find(PMIN);
tree = [src dst];
dlmwrite('tree-conncomps.txt',tree,'delimiter',' ','precision','%6.0f');

pause
bioc;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% TREE-4: MST TREE %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(sum(whichgrphs==2)>0)
tStart = tic;
[SP, TREEMIN, trash, roots] = findMST(markedpaths, markedset); % TREEMIN is kxk where k = |markedset|
tMSTelapsed = toc(tStart)

[trash, PMIN, trash, trash, cost] = expandPathsAll(SP, TREEMIN, markedpaths, markedset, markedset, W, 1);

%PMIN
fprintf('Cost-alg from MST TREE is %f \n', cost+length(roots)*log2(N));
cost = findTreeCost( PMIN, markedset, markedset(roots), deg );
fprintf('Cost-act from MST TREE is %f \n', cost);
%cost = findTreeCostJ( PMIN, markedset, markedset(roots), deg );
%fprintf('Cost-J from MST TREE is %f \n', cost);

% write tree-4
[src dst trash] = find(PMIN);
tree = [src dst];
dlmwrite('tree-arb.txt',tree,'delimiter',' ','precision','%6.0f');

pause
bioc;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% TREE-2: LEVEL-1 TREE %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(sum(whichgrphs==3)>0)
tStart = tic;
[trash, L1MINP, rootsp, mincost] = findMinLevel1(markedpaths, markedset, W, deg, 1);
tL1elapsed = toc(tStart)

[trash, PMIN, trash, trash, cost] = expandPathsAll(SP, L1MINP, markedpaths, markedset, markedset, W, 1);

%PMIN
fprintf('Cost-alg from LEVEL-1 TREE is %f \n', cost+length(rootsp)*log2(N));
cost = findTreeCost( PMIN, markedset, markedset(rootsp), deg );
fprintf('Cost-act from LEVEL-1 TREE is %f \n', cost);
%cost = findTreeCostJ( PMIN, markedset, markedset(rootsp), deg );
%fprintf('Cost-J from LEVEL-1 TREE is %f \n', cost);

% write tree-2
[src dst trash] = find(PMIN);
tree = [src dst];
dlmwrite('tree-level1.txt',tree,'delimiter',' ','precision','%6.0f');

pause
bioc;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% TREE-3: LEVEL-2 TREE %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(sum(whichgrphs==4)>0)
  
tStart = tic;
[trash, SPL1, trash, minL2MIN, mintreenodes, minrootsL2, costL2, costNaive, trash] = findMinLevel2(markedpaths, markedset, W, deg, 1, 0);
tL2elapsed = toc(tStart)

fprintf('Cost-alg from LEVEL-2 TREE is %f \n', costNaive);
fprintf('Cost-act from LEVEL-2 TREE is %f \n', costL2);  
%fprintf('Cost-J from LEVEL-2 TREE is %f \n', costTJ); 

[P PMIN trash, trash, cost] = expandPathsAll(SP, minL2MIN, markedpaths, mintreenodes, markedset, W, 1);
cost+length(minrootsL2)*log2(N)



% write tree-3
[src dst w] = find(PMIN);
tree = [src dst];
dlmwrite('tree-level2.txt',tree,'delimiter',' ','precision','%6.0f');

pause
bioc; close all;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% write the candidate graph %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[src dst trash] = find(P);
cg = [src dst];
dlmwrite('candidate-graph.txt',cg,'delimiter',' ','precision','%6.0f');





end




